
# q-expansion of newform 144.12.a.i, downloaded from the LMFDB on 05 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 144
weight = 12
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [0], [268], [0], [-134374], [0], [4655368], [0], [0], [249734764], [-782919730], [0], [1549433416], [0], [0], [0], [4430940374], [-15458751248], [0], [19805520230], [-32885832404], [0], [0], [-112637211442], [0], [73068123460], [0], [-279512328742], [0], [-394909570460], [0], [0], [-1149471836816], [0], [-1879586495156], [-561914369002], [-1045396961144], [0], [0], [0], [-5027869824094], [0], [-5298284466814], [0], [1893796753492], [-1606477746704], [-11850168871436], [0], [8683051940882], [0], [0], [22909927397294], [0], [0], [0], [0], [47262930283828], [22915186739042], [0], [58114699342624], [0], [83713832158192], [0], [94619031791834], [0], [-97999974916832], [70202549103422], [0], [170908703822486], [0], [0], [-232887630238292], [-111920359837666], [231781075129288], [0], [0], [-381008360761018], [0], [-352669621300618], [0], [-32098958938222], [0], [-319591189611934], [-670251992564588], [0], [0], [441639496391270], [0], [-753453187160540], [0], [0], [-784925958498308], [0], [827787857100832], [0], [0], [0], [1673023450587976], [158500984058954], [2196042785816920], [0], [0], [0], [1670384134969456], [3063478459244498], [0], [0], [0], [517176375658826], [-4033597504882460], [3370117000017230], [0], [4456695662024992], [-3702574407997124], [0], [3013890812885080], [0], [0], [0], [-1775083064310562], [-7062668602651150], [0], [0], [8232789757328392], [0], [-9537098821055278], [0], [-9622345593772868], [-4237415493016150], [1721822324115184], [0], [-12083969207738948], [10454396622565922], [0], [-5763602855053822], [0], [-13175847688046792], [0], [0], [8648173163876344], [0], [1890774381096796], [0], [-15433139075348854], [0], [-6858432242621314], [0], [27073814202747256], [0], [13162592902057862], [0], [31784978935244296], [0], [-36635156352255080], [0], [37439554460856028], [0], [43938624301290602], [0], [0], [0], [-23334532738726460], [0], [0], [0], [47623203895699036], [61221317012168270], [-49820983555410082], [0], [0], [-46330873257391786], [0], [-14332360591431754], [61029340479351868], [0], [-16512667485578816], [0], [87232472835625684], [-83483155261147162], [-61959054703502612], [0], [-57341286930631726], [0], [0], [0], [102293410599418058], [-86528712448204256], [-39686755107308458], [0], [-12251132476551874], [0], [-100322032216348736], [0], [0], [0], [-54060347721731998], [-105927063651136150], [0], [0], [0], [182847247863475732], [-95092911463898110], [-174548389758270514], [0], [0], [234725593386665332], [0], [0], [-101290378029289736], [257162681543147378], [0], [219683927305980916], [0], [0], [-69678194574529222], [-170321569561856852], [0], [0], [0], [193197699487479794], [-6560693654645132], [0], [83263883645895460], [0], [-86640112678300162], [0], [0], [423300440298515812], [0], [-140525342353859494], [184570828266675256], [40070747025105796], [0], [150661138201757080], [0], [-538630167010211266], [0], [0], [0], [0], [589341826667975104], [-683506072847985764], [368191609740648998], [0], [0], [147782853857012572], [0], [-770851062207880928], [0], [-758337809951891350], [0], [0], [441665099009548598], [0], [0], [412151724143307746], [-137271950786081840], [0], [-941181545539121110], [-1025547098498785964], [0], [109475741785579826], [-686838843263641546], [0], [402904748308351936], [0], [0], [862185353301389080], [0], [-1328623079945999674], [1354084857145326160], [1341728501832684230], [-1240382013712213892], [-1441982258120586322], [828693658725271444], [0], [1421146735305021302], [1277184169787446874], [0], [0], [-1102782114051400268], [0], [1220764691542185326], [1950661012409262328], [0], [1802253456122666894], [0], [-987233793925908404], [0], [0], [0], [0], [0], [-1729681753238276314], [0], [2300925327744271876], [0], [0], [2537177416655288920], [-1314063834107806390], [0], [-1475841793321302212], [0], [-1584217806922383512], [-2710197664956137698], [0], [-749105050043396902], [0], [3099525253667049230], [0], [0], [0], [-3519128398273214936], [0], [3636543275325419222], [0], [0], [-1679646810106619986], [0], [-2770929303482788832], [2905351751472869498], [0], [-1369764481119643316], [0], [-4313998937793362338], [-3775353436538274008], [3929463789922689736], [0], [0], [-3514850188481196934], [0], [3033289587065764948], [0], [-5250415876100318864], [0], [-5258934911481334858], [0], [1404062909456378246], [4392807822200005540], [4650808306151829566], [0], [0], [3591583004792881636], [0], [0], [1029175282311307634], [-6876402673762697792], [0], [0], [3212688661544448184], [6692293286068967270], [0], [435213971856264244], [7219401627568054622], [0], [0], [0], [0], [0], [6832280130770143982], [0], [0], [0], [-6960445242806705840], [5842495284900614714], [0], [9125726582090982700], [-2635754588263676854], [0], [8155748976654785608], [0], [0], [3587577079771113052], [-9796414520907656182], [0], [0], [-11866023179219208286], [0], [12557604616972386662], [0], [0], [-11572300505542807700], [0], [422969347127331256], [0], [-13943203149607389284], [13511122911002445890], [-3995402665680154160], [0], [-10100896179966729922], [0], [0], [-14122459341443145368], [0], [12592900135161665654], [-2339757167691796748], [0], [-7265166800336805464], [0], [-16383656142520425514], [0], [-8843478555552411068], [0], [0], [-14483024215784249492], [-9106990349511400198], [0], [17940960799492915624], [0], [-5731423569954423166], [0], [0], [16349401593031924456], [-2859159197766789562], [0], [0], [-8581791249081895412], [0], [0], [0], [12100013091031200878], [0], [0], [24108512328658129610], [0], [0], [0], [-8624859819509601416], [0]]
